use strict;
use warnings;

use Module::Build;
use ExtUtils::PkgConfig;

my %pkg_info = ExtUtils::PkgConfig->find( 'parrot' );

my $class = Module::Build->subclass(
	code => <<'END_HERE',
	use Cwd;

	sub ACTION_build
	{
		my $self    = shift;
		my $old_dir = cwd();
		chdir 't';

		system(qw( parrot -o greet.pbc greet.pir ) ) == 0
			or die "Cannot compile PBC for test: $?";

		chdir $old_dir;
		$self->SUPER::ACTION_build( @_ );
	}
END_HERE
);

my $builder = $class->new(
    module_name         => 'Parrot::Embed',
    license             => 'perl',
    dist_author         => 'chromatic <chromatic@wgz.org>',
    dist_version_from   => 'lib/Parrot/Embed.pm',
    build_requires => {
        'Test::More' => 0,
    },
    add_to_cleanup      => [ 'Parrot-Embed-*' ],
	extra_compiler_flags => $pkg_info{cflags},
	extra_linker_flags   => $pkg_info{libs},
);

$builder->create_build_script();
